/*
The I-8014W contains two types of Magic Scan . One is type 1: standard (each sample clock only samples a single. ) 
and Type 2: virtual Sample and hold (each sample clock will to sample all scan channels that have been set. )
Both of two types can set as polling Mode,Internal Hardware trigger Mode and External Hardware trigger Mode
This Demo will show you how to use Magic Scan ,step by step

Step1: 
    ret=i8014W_Init(slot); , this function will check hardware ID, 
	if retrun code ret is -1 means this slot may empty or wrong I/O (not I-8014W) 
	,return 0 this slot is I-8014W 	
	
Step2: 
    i8014W_ConfigMagicScan( slot, chArr, gainArr, scanChCount, sampleRate, scanMode, triggerSource, triggerState , & realsampleRate);
    for Configure Magic Scan Parameters and return real sample rate that used by I-8014 module	

Step3:
	i8014W_StartMagicScan(int slot);
	to start Magic Scan function 
	(If i8014W_ConfigMagicScan use external trigger signal mode , module will not start to save data .
	It will start to scan data when the signal is active) 
	
Step4:
    Send a external trigger signal for I-8014 module to Start magic scan 
    (external trigger signal Mode only)

Step5:
	to read the data from FIFO in main loop
    i8014W_ReadFIFO(int slot, short hexData[], short readCount,short* dataCountFromFIFO);
    

Step6:
	
	i8014W_StopMagicScan( slot);
	stop Magic Scan function 

Step7:
	to show calibrated analog data.
	When we use Magic scan to read read data from FIFO ,the analog data is not calibrated 
	We can use following function to get the calibrated data.
	void i8014W_CalibrateDataHex(int slot, short iGain,short dataFromFIFO, short* calibratedAI);
	void i8014W_CalibrateData(int slot, short iGain,short dataFromFIFO, float* calibratedAI);
*/


#include <stdio.h>
#include <stdlib.h>
#include "8000E.h"
#include "8014W.h"
#include <memory.h>
#include <math.h>
int slotIndex=-1;
short jumper;

unsigned long T1,T2; 

int chArr[16], gainArr[16], scanChCount;
float sampleRate,realsampleRate;
int scanMode, triggerSource, triggerState;
short rawData[8192];
long TargetCnt=100;
long totalScanned=0;
char GainStr[5][32]={"+/-10V","+/-5V","+/-2.5V","+/-1.25V","+/-20mA"};
char ModeStr[2][32]={"Standard Mode", "Sample and Hold Mode"};
char SourceStr[3][32]={"Software Command", "Internal Interrupt Signal","External Trigger Signal"};
char StateStr[2][32]={"High","Low"};
int runFlag=1;
int selGain;
int showCnt=0;
void ShowAI(void);

void main()
{
	int slot,ch,c;
	short i,ret;
	int maxCh=8;        
    char sTemp[20];               
	long readCnt=0;	    
    
    
    InitLib(); // Call this function initialize the lib    
    
    Print("\n\n This Demo will show how to use magic scan function to read analog input\n\n");
	
	Print(" Search I-8014W ....\n");	           	        	
	for(slot=0;slot<8;slot++)
	{
		
		if(i8014W_Init(slot)==0) //find I-8014W module out
		{				
			slotIndex=slot;			
			break;
		}
	}		
	if(slotIndex==-1)
	{
		Print("\t There is no i8014 at i8000\n");
		DelayMs(100);
		exit(0);
	}
	else
	{
		Print("\t There is an i8014 at slot %d\n",slotIndex);
	}
      
    jumper= i8014W_GetSingleEndJumper(slotIndex);
	if(jumper)
	{
		maxCh=16;
        Print("\t i8014W Input Mode=Single-End and can have maximum 16 analog input\n\r");
    }
    else
    {
    	maxCh=8;
        Print("\t i8014W Input Mode=Differential and can have maximum 8 analog input\n\r");
    }
    
    
	// define the sampling channels array chArr[] 
	// Differential Mode: element for chArr[] range from channel 0 ~ 7 ,
	// Single ended Mode: element for chArr[] range from channel 0 ~ 15 
	// max element counts for chArr[] can be 16 
	// define gain type array for each channel		
	// gainArr[] element for gainArr[] range from channel 0 ~ 15
	// Gain value : 0="+/-10V",1="+/-5V",2="+/-2.5V",3="+/-1.25V",4="+/-20mA" 
	// max element counts for gainArr[] can be 16 
start:	
		
	Print ("\n\n Input all i8014W_ConfigMagicScan parameters :\n\r");

    
    Print("\n Step 1: Define scaned channel counts for magic scan:");
    
	Print("\n Input scaned channel counts (1~16) :");
	LineInput(sTemp,20);
    scanChCount=atoi(sTemp);
    
    Print(" Now we have scaned channel counts = %d\n\n",scanChCount);
    
   // assing input range (gain)
	Print("\n Step 2: Define input range (gain)");      
    Print("\n The Gain definition of I-8014W");
	for(i=0;i<5;i++)
    {
    	Print ("\n\t Select %d :  %s",i,GainStr[i]);
    }
    Print("\n\r");

	Print (" Select which Gain of (0~4):",i);
	LineInput(sTemp,20);
	selGain = atoi(sTemp);
    for(i=0;i<scanChCount;i++)
    {    	   	
    	chArr[i]=i;    		
    	gainArr[i]=selGain; 		
    }


	// Sample channel counts
	// scanChCount can be 1 ~ 16 (max =16 for both Differential and Single Ended)

	Print("\n\n Step 3: Define Sample Rate of I-8014W\n");
	Print (" Input sample rate less than 100Hz will be helpful to understand how i8014W_ReadFIFO works");
    // Sample rate : 2 ~ 250KHz  
    Print (" Input Sample rate of 8014W (2~2500000) :");
    LineInput(sTemp,20);
    sampleRate=atoi(sTemp);
    Print(" Note: the real sample rate may not the same as user input");
	Print("\n the function i8014W_ConfigMagicScan return code is the \n real sample rate accepted by I-8014W\n");
    
    
    
    // Scan Mode = 1 M1 standard mode, Each Sample clock only samples a single channel
    // Scan Mode = 2 M2 Sample and hold Mode
    Print ("\n\n Step 4:Select Scan Mode of I-8014W:\n");
    Print ("\t Scan Mode 1= M1 Standart Mode \n\t Scan Mode 2= M2 Sample and Hold Mode \n\r");
    Print (" Input Scan Mode of 8014W (1 or 2) :");
    LineInput(sTemp,20);
    scanMode=atoi(sTemp);
    
    
    // trigger source 0, start magic scan via polling Mode, software command can only trigger one slot I-8014
    // trigger source 1, start magic scan via internal interrupt signal Mode, internal interrupt signal can trigger all I-8014Ws on slots at the same time.
    // trigger source 2, start magic scan via external trigger signal Mode, extern trigger can also trigger all I-8014Ws on slots at the same time.
    Print ("\n\n Step 5: Select Trigger Source of I-8014W,\n I-8014W can have 3 types of trigger source \n");
    Print ("\t trigger source 0= Software Command \n\t trigger source 1= Internal Interrupt Signal \n\t trigger source 2= External Tigger Signal\n\r");
    Print (" Input trigger source of 8014W (0~2) :");
    LineInput(sTemp,20);
    triggerSource=atoi(sTemp);
    
    
    
    // triggerState only related with external DI trigger signal.
    // if triggerState 0, external DI trigger signal must be high to start magic scan (default)
    // if triggerState 1, external DI trigger signal must be low to start magic scan
    Print ("\n\n Step6: Select Trigger State of I-8014W if select external trigger source.\n");
    if(triggerSource==2)
    {	    
	    Print ("\t trigger State 0= external trigger singal high to start magic scan \n\t trigger State 1= external trigger signal low to start magic scan\n\r");
	    Print (" Input trigger State of I-8014W (0 or 1) :");
	    LineInput(sTemp,20);
	    triggerState=atoi(sTemp);
	}
	else
	{
		Print ("\t Not external trigger source, trigger state =0 \n");
		triggerState=0;
	}
    
    
    
    // When Configure Sample Rate of Magic Scan, the result sample rate may be different from input sample rate 
    // Note i8014W_ConfigMagicScan only tell I-8014W how to start magic scan,it will not start magic scan
    // We have to call i8014W_StartMagicScan to start up the magic scan.
	i8014W_ConfigMagicScan(slotIndex,chArr,gainArr,scanChCount, sampleRate, scanMode,triggerSource,triggerState, &realsampleRate);	
	
	
	Print("\n\nThe Magic Scan Configurations of I-8014W are:\n");
	Print("\t Scan channel count = %d\n", scanChCount);
	for(i=0;i<scanChCount;i++)
	{
		Print("\t CH[%d]= %d\t Gain[%d]= %d ( %s )\n",i, chArr[i],i, gainArr[i], GainStr[gainArr[i]]);
	}
	Print("\t Scan Mode = %d ( %s )\n", scanMode, ModeStr[scanMode-1]);
	Print("\t Trigger Source = %d ( %s )\n", triggerSource, SourceStr[triggerSource]);
	if(triggerSource==2)
		Print("\t Trigger State = %d ( %s )\n", triggerState, StateStr[triggerState]);
	else
		Print("\t Trigger State = %d ( No need for External Trigger Signal )\n", triggerState);
	Print("\t Set Sample Rate = %6.3f  Real Sample Rate = %6.3f \n",sampleRate, realsampleRate);
	
	
	
	Print("Press any Key to Start magic scan \n\r");
	
	Getch();
	//Send external trigger for I-8014W module
	
	if(triggerSource==2)
	{
		Print("Wait for external trigger signal source to %s .....\n", StateStr[triggerState]);
	}
	else
	{
		Print("Wait for Magic Scan .....\n");
	}
	i8014W_StartMagicScan(slotIndex);	
				
     T1=GetTimeTicks();     
     totalScanned=0;	  
     
      	
 	// i8014W_ReadFIFO is a non block API function
 	// rawData+totalScanned totalScanned act as pointer index of rawData array
 	// TargetCnt-totalScanned means the remaining data count according to total data want to sample
 	// readFromFIFO means the data count read from FIFO for each i8014W_ReadFIFO read action
 	// It is better to run this demo with low frequency(for example 10Hz) then we can know how this function works.
	ret= i8014W_ReadFIFO_BlockMode(slotIndex,rawData, TargetCnt ,&readCnt);// <=== get data from FIFO ,long* dataCnt
	i8014W_StopMagicScan(slotIndex);     
	T2=GetTimeTicks()-T1; // total time for magic scan to polling data
	Print("Magic scan sample %ld data  spend time = %lu ms  \n\n\n",readCnt, T2); 

	if(ret ==FIFO_LATCHED)
	{
		Print("Magic Scan FIFO Latched, Sample Rate too high , try to use Lower Sample Rate\n");
		
		i8014W_UnLockFIFO(slotIndex);
		i8014W_ClearFIFO(slotIndex);

		goto start;
	}
	if(readCnt>0)
	{
		if(TargetCnt != readCnt)
		{
			Print("Magic Scan Block Mode Error, TargetCnt %ld must be the same as readCnt %ld\n");
			Print("try again \n");
			goto start;
		}
		totalScanned = readCnt;

	    
	}
	
	Print("Magic scan total spend time = %lu ms  \n\n\n",T2); 
	
	Print("Press 's' or 'S' to Show AI, others to next step\n\r");
	c=Getch();	
	if(c=='s' || c=='S')
		ShowAI();
		
	Print("Press 'q' or 'Q' to exit program, others to Start magic scan again!\n\r");
	
	c=Getch();	
	if(c!='q' && c!='Q')
		goto start;  // continue another test
	else
		exit(0); // end of program
	
}

// 
void ShowAI(void)
{
	int i,c;
	float calibratedAI=0;
	
	Print("If want to save result to PC, press 'Alt + C' and input 'L1 log.csv'\n");
	Print("Press any key to Start to Print all data:\n\n\r");
	c=Getch();	
	for(i=0;i<totalScanned;i++)	
	{		
		i8014W_CalibrateData(slotIndex, gainArr[i %scanChCount],rawData[i],&calibratedAI);
		Print("%d,%d,%5.4f\n",i,i%scanChCount,calibratedAI);			
	}
	Print("\nPress 'Alt + C' and input L0 to stop log operation");
}
