/************	
	Author: Martin Hsu
	Email: service@icpdas.com
	Date 2007/11/6
	
	i8088 is a PWM module with 8 DI Pins can be defined as hardware trigger	pin
	it also can set PWM as synchrous mode
	
	[Normal PWM pulse define as duty]
	1. i8088W_InitLib(slot)
	2. i8088W_SetPWMCountMode(slot,ch,countMode); //0 burst; 1 continous
		if burst mode
			i8088W_SetBurstCount(slot,ch,burstCount);
	3. i8088W_SetPWMDuty(slot,ch,hz,duty);
	4. i8088W_PWM_Start(slot,ch);

	[Normal PWM pulse define pulse width]
	1. i8088W_InitLib(slot)
	2. i8088W_SetPWMCountMode(slot,ch,countMode); //0 burst; 1 continous
		if burst mode
			i8088W_SetBurstCount(slot,ch,burstCount);
	3. i8088W_SetPWMDutyCount(slot,ch,hz,highDutyCount,lowDutyCount);
	4. i8088W_PWM_Start(slot,ch);
	

	[Synchrous PWM pulse define as duty]
	1. i8088W_InitLib(slot)
	2. i8088W_SetPWMCountMode(slot,ch,countMode); //0 burst; 1 continous
		if burst mode
			i8088W_SetBurstCount(slot,ch,burstCount);
	3. i8088W_SetPWMDuty(slot,ch,hz,duty);
	4. i8088W_SetSyncChannel(slot,ch,enBit);//enBit=1 to set as synchrous channel
	5. i8088W_SetSyncStart(slot);
	
	[Hardware trigger PWM pulse define as duty]
	1. i8088W_InitLib(slot)
	2. i8088W_SetPWMCountMode(slot,ch,countMode); //0 burst; 1 continous
		if burst mode
			i8088W_SetBurstCount(slot,ch,burstCount);
	3. i8088W_SetPWMDuty(slot,ch,hz,duty);
	4. i8088W_SetHardwareTrigChannel(slot,ch,enBit);//enBit=1 to set as start flag; enBit=0 to set as stop flag.
	5. The wait the hardware DI pin trigger signal to start or to stop the PWM pulse;	
	
#define OK 		 				0	
#define ID_ERROR 				-1
#define SLOT_OUT_RANGE			-2
#define CHANNEL_OUT_RANGE		-3
#define SELECT_CHANNEL_ERROR 	-4
#define HI_DUTY_OUT_RANGE		-5
#define LO_DUTY_OUT_RANGE		-6
	
*******************************/




/********************************
	i8088 Error Code definition	
*******************************/




#include "..\..\Lib\8000E.h"
#include "..\..\Lib\8088W.h"

int slotIndex=-1;
int runFlag=1;
int startMode=0;
char title[]="Demo for i-8088 PWM";
void showMenu(int step)
{
    
    Print("\n\n****************************************************\n\r");    
    switch(step)
    {
    	case 1:
    		Print("Enter '1' Step 1 to set PWM mode.\n\r");
    	break;
    	case 2:
    		Print("Enter '2' Step 2 to set PWM duty.\n\r");
    	break;
    	case 3:
    		Print("Enter '3' Step 3 to start PWM.\n\r"); 
    	break;
    	case 4:
    		Print("Enter '4' Step 4 to stop PWM.\n\r"); 
    	break;    	
    }
 	Print("Enter 'Q' or 'q' to Exit the program.\n\r");
}
void Find_i8088W(void)
{
	int i,id,Base;
	
	
	for (i=0; i<GetNumberOfSlot(); i++)
	{
	    slotIndex=i;
	    
	    id=NameOfModule[i];
	    if (id==88) 
	    {
	    	Print("There is a i-8088 located at Slot %d\n",slotIndex);	
	    	return;    
	    }
	}
	slotIndex=0xff;
	Print("There is no i-8088 Found\n");	
}
int main(void)
{
	int ch;
    char c,c1,c2;
    unsigned long hz;
    unsigned long realDeciHz=0;
    unsigned int realDeciDuty=0;
    unsigned int cnt,duty,flag,flag2;
    float fhz,fduty;
    
    char buf[32];
    

    
    InitLib();
    Find_i8088W();
    Print("\n****************************");
    Print("\n%s",title);
    Print("\nLattice Version =: %04X",i8088W_GetFirmwareVersion(slotIndex));
    Print("\nLibrary Version =: %04X",i8088W_GetLibVersion());
    i8088W_GetLibDate(buf);
    Print("\nBuild Date =: %s",buf);
    Print("\n****************************\n");
    
    
    c='1';
    while(runFlag)
    {
        switch(c)
        {
            case '1': // Set PWM Mode   
            // If set burst count = 1000, when call i8088W_PWM_Start, it will produce 1000 pulse count then stop PWM output.
            // call i8088W_PWM_Start will produce 1000 pulse again. 
            // If set coutinuous mode, when i8088W_PWM_Start, it will produce coutinous pulse, if call i8088W_PWM_Stop ,PWM stop.          
                Print("\n\n******** ~ SET PWM MODE ~ **********\n\r");  
                Print("Enter '0' to Set Burst Count\n");
                Print("Enter '1' to Set Continous\n");
                Print("Enter 'Q' or 'q' to Exit the program.\n\r");
                c1=Getch();
                 switch(c1)
        		{
        			case '0':
        				Print("Choice Burst Count Mode\n");
        				Print("Please Input Burst Count 0~65535\n");
        				Scanf("%u",&cnt);
        				for(ch=0;ch<8;ch++)
        				{
			    			i8088W_SetPWMCountMode(slotIndex,ch,0);
			    			i8088W_SetBurstCount(slotIndex,ch,cnt);
			    		}
			    		c='2';	  
			    	break;
			    	case '1':
			    		Print("Choice Continous Mode\n");
			    		for(ch=0;ch<8;ch++)
			    		{
			    			i8088W_SetPWMCountMode(slotIndex,ch,1);
			    			i8088W_SetBurstCount(slotIndex,ch,1);	
			    		}	
			    		c='2';	  	    	
			    	break;
		            case 'Q':case 'q':
		                runFlag= 0;
		            break;			    	
			    }
                     
            break;
            case '2': //Set PWM Duty
                Print("\n\n******** ~ SET PWM DUTY ~ **********\n\r");   
                Print("Enter '0' to Set Normal integer Duty,for example 50 means 50% duty\n");
                Print("Enter '1' to Set x10 integer Duty,for example 500 means 50.0% duty\n");
                Print("Enter '2' to Set float Duty,for example 50.0 means 50.0% duty\n");
                Print("Enter 'Q' or 'q' to Exit the program.\n\r");
                c1=Getch();
                 switch(c1)
        		{
        			case '0':
        				Print("Choice Set Normal integer Duty\n");
        				Print("First Input Hz 1~500000\n");
        				Scanf("%lu",&hz);        			
        				Print("Then Input duty 1~99\n");
        				Scanf("%u",&duty);
        				
        				
        				for(ch=0;ch<8;ch++)
        				{
			    			i8088W_SetPWMDuty(slotIndex,ch,hz,duty);
			    			
					    	
					    }
					    i8088W_GetRealPWMDuty_Deci(slotIndex,0, &realDeciHz, &realDeciDuty);
					    Print("The PWM will be %lu Hz with %d % Duty\n",realDeciHz/10,realDeciDuty/10);
					   
					    c='3'; 	    	    		
			    	break;
			    	case '1':
        				Print("Choice Set x10 integer Duty\n");
        				Print("First Input x10 Hz 1~5000000\n");
        				Scanf("%lu",&hz);        			
        				Print("Then Input x10 duty 1~999\n");
        				Scanf("%u",&duty);
        				fhz=((float)hz)/10;
        				fduty=((float)duty)/10;
        				Print("The PWM will be %.1f Hz with %.1f % Duty\n",fhz,fduty);
        				for(ch=0;ch<8;ch++)
        				{
			    			i8088W_SetPWMDuty_Deci(slotIndex,ch,hz,duty);					    	
					    } 
					    i8088W_GetRealPWMDuty_Deci(slotIndex,0, &realDeciHz, &realDeciDuty);
					    Print("The PWM will be %lu Hz with %d % Duty\n",realDeciHz,realDeciDuty);
					    c='3'; 
					       
		                			    		  	
			    	break;
			    	case '2':
			    		Print("Choice Set float Duty\n");
        				Print("First Input x.1 Hz 0.1~500000.0\n");
        				Scanf("%f",&fhz);        			
        				Print("Then Input x.1 duty 0.1~99.9\n");
        				Scanf("%f",&fduty);
        				Print("The PWM will be %.1f Hz with %.1f % Duty\n",fhz,fduty);
        				for(ch=0;ch<8;ch++)
        				{
			    			i8088W_SetPWMDuty_Float(slotIndex,ch,fhz,fduty);					    	
					    }	
					    i8088W_GetRealPWMDuty_Deci(slotIndex,0, &realDeciHz, &realDeciDuty);
					    Print("The PWM will be %.f Hz with %.f % Duty\n",(float)(realDeciHz/10),(float)(realDeciDuty/10));
					    c='3'; 
					  
		                		    			    	
			    	break;
		            case 'Q':case 'q':
		                runFlag= 0;
		            break;			    		
			    	default:
			    		
			    	break;	    	
			    }   
                		    

            break;
            case '3':	//start PWM
                Print("\n\n******** ~ START OR STOP PWM ~ **********\n\r");  
                Print("Enter '0' to Set Normal PWM\n");                
                Print("Enter '1' to Set Hardware Trigger PWM\n");
                Print("Enter '2' to Set Synchronous PWM\n");
                
                Print("Enter '3' to Set PWM Mode\n");                
                Print("Enter 'Q' or 'q' to Exit the program.\n\r");
                c1=Getch();
                 switch(c1)
        		{
        			case '0':  
        				//startMode=0; 
		                Print("\n\n******** ~ START OR STOP Normal PWM ~ **********\n\r");  
		                Print("Enter '0' to Stop Normal PWM\n");
		                Print("Enter '1' to Start Normal PWM\n"); 
		                Scanf("%u",&flag);  
		                if(flag==1)
		                {       				
	        				for(ch=0;ch<8;ch++)
	        				{     				      				
						    	i8088W_PWM_Start(slotIndex,ch); 
						    	DelayMs(8);
						    }
						    Print("PWM Start\n");  
						}
						else
						{
	        				Print("PWM Stoped\n");	
	        				for(ch=0;ch<8;ch++)
	        				{   
			    				i8088W_PWM_Stop(slotIndex,ch); 
			    			}							
						}						
		                c='3';			    		
			    	break;
			    	case '1':
		                Print("\n\n******** ~ Configure DI pin as PWM Trigger Signal  ~ **********\n\r");  
		                Print("Enter '0' to set DI as normal DI (Disable Hardware Trigger function)\n");
		                Print("Enter '1' to set DI as external trigger signal to start PWM , if DI high, PWM start \n"); 
		                Print("Enter '2' to set DI as external trigger signal to stop PWM, if DI high, PWM stop \n"); 
		                flag=Getch();
		                if(flag=='0')
		                { 
		                	Print("Hardware Triggered PWM Stoped\n");	
		    				for(ch=0;ch<8;ch++)
		    				{
				    			i8088W_SetHardwareTrigChannel(slotIndex,ch,0);	  
				    		}		
	        				
						}
						else if(flag=='1')
						{
			    			for(ch=0;ch<8;ch++)
	        				{	        					
				    			i8088W_SetHardwareTrigChannel(slotIndex,ch,1);	  
				    		}					    
						    Print("\n ...\n ..\n DI wait external trigger signal to start PWM\n");  					
						}
						else if(flag=='2')
						{
							
		    				for(ch=0;ch<8;ch++)
		    				{		    					
				    			i8088W_SetHardwareTrigChannel(slotIndex,ch,2);	  
				    		}		
							Print("\n ...\n ..\n DI wait external trigger signal to start PWM\n"); 	
						}
						
					    c='3'; 	    		    			    	
			    	break;	
			    	// When Configure PWM as synchronize output for each channel,
			    	// i8088W_Sync_Start will start the PWM and synchronize the rising edge for configured channels at the same time.
			    	// if PWM is sending pulse , calling i8088W_Sync_Start will just synchronize the rising edge for configured channels at that moment		    				    	
			    	case '2':  
		                Print("\n\n******** ~ Enable/Disable Synchronous PWM ~ **********\n\r");  
		                Print("Enter '0' to Disable Synchronous PWM\n");
		                Print("Enter '1' to Enable Synchronous PWM\n"); 
		                flag=Getch(); 
		                if(flag=='1')
		                {       				
	        				for(ch=0;ch<8;ch++)
	        				{
				    			i8088W_SetSyncChannel(slotIndex,ch,1);	 
				    		} 
			    			Print("Enable Synchronous mode\n\n"); 
			                Print("\n\n******** ~ Stop/Start Synchronous PWM ~ **********\n\r");  
			                Print("Enter '0' to Stop Synchronous PWM\n");
			                Print("Enter '1' to Start Synchronous PWM\n"); 
			                flag2=Getch();
			                if(flag2=='1')
			                {
			                	Print("Start Synchronize PWM\n");
			                	i8088W_Sync_Start(slotIndex); 
			                }
			                else if(flag2=='0')
			                {
			                	Print("Stop Synchronize PWM\n");
			                	i8088W_Sync_Stop(slotIndex); 	
			                }			    			
				    		
				    	}
				    	else
				    	{
				    		Print("Disable Synchronous mode\n");
	        				for(ch=0;ch<8;ch++)
	        				{				    		
				    			i8088W_SetSyncChannel(slotIndex,ch,0);				    			
				    		}
				    		 
				    	}				    		
					    
					     
					    c='3';  		    		  	
			    	break;

        			
		    		case '3':
		    			c='1';
		    		break;		    	
		            case 'Q':case 'q':
		                runFlag= 0;
		            break;			    	
			    	
			    	default:
			    		c='3';
			    	break;	    	
			    }   
            break;                 
            
            case 'Q':case 'q':
                runFlag= 0;
            break;
        }    
    }    
    return 0;
}









